
#include <SoftwareSerial.h>

SoftwareSerial mySerial(10, 11); // RX, TX

void setup()  
{
  Serial.begin(9600);
  mySerial.begin(9600);
}

void loop()
{                                                                                                                                                                
  for(int out = 1; out <= 4; out++)
  {
    for(int in = 1; in <= 4; in++)
    {
      Serial.print("out: " + String(out) + "in: " + String(in));
      switchMatrix(out,in);
      delay(4000);
    }
  }
}

/*
  Startech's VS440HDMI Matrix Switch follows a 5-byte protocol
 for controlling the matrix input-output via serial comm.
 
 Example.
 Switching output = 1, input = 1: 20 01 01 01 41... 
 20 - device byte.
 01 - request byte to switch video.
 01 - output byte.
 01 - input byte.
 41 - crc equivalent.
 
 List of commands we 'sniffed out'.
 
 --OUT 1--
 IN 1: 20 01 01 01 41
 IN 2: 20 01 01 02 A3
 IN 3: 20 01 01 03 FD
 IN 4: 20 01 01 04 7E
 
 --OUT 2--
 IN 1: 20 01 02 01 14
 IN 2: 20 01 02 02 F6
 IN 3: 20 01 02 03 A8
 IN 4: 20 01 02 04 2B
 
 --OUT 3--
 IN 1: 20 01 03 01 D0
 IN 2: 20 01 03 02 32
 IN 3: 20 01 03 03 6C
 IN 4: 20 01 03 04 EF
 
 --OUT 4--
 IN 1: 20 01 04 01 BE
 IN 2: 20 01 04 02 5C
 IN 3: 20 01 04 03 02
 IN 4: 20 01 04 04 81  
 
 */
void switchMatrix(int output, int input)
{
  mySerial.write(0x20); // write device ID. (1st byte)
  mySerial.write(0x01); // requests for video switch. (2nd byte)

  if(output == 1)
  {
    mySerial.write(0x01); // output number. (3rd byte)
    if(input == 1)
    {
      mySerial.write(0x01); // input number. (4th byte)
      mySerial.write(0x41); // crc equivalent. (5th byte)
    }
    else if(input == 2)
    {
      mySerial.write(0x02);
      mySerial.write(0xA3);
    }
    else if(input == 3)
    {
      mySerial.write(0x03);
      mySerial.write(0xFD);
    }
    else if(input == 4)
    {
      mySerial.write(0x04);
      mySerial.write(0x7E);
    }
  }
  else if(output == 2)
  {
    mySerial.write(0x02);
    if(input == 1)
    {
      mySerial.write(0x01);
      mySerial.write(0x14);
    }
    else if(input == 2)
    {
      mySerial.write(0x02);
      mySerial.write(0xF6);
    }
    else if(input == 3)
    {
      mySerial.write(0x03);
      mySerial.write(0xA8);
    }
    else if(input == 4)
    {
      mySerial.write(0x04);
      mySerial.write(0x2B);
    }
  }
  else if(output == 3)
  {
    mySerial.write(0x03);
    if(input == 1)
    {
      mySerial.write(0x01);
      mySerial.write(0xD0);
    }
    else if(input == 2)
    {
      mySerial.write(0x02);
      mySerial.write(0x32);
    }
    else if(input == 3)
    {
      mySerial.write(0x03);
      mySerial.write(0x6C);
    }
    else if(input == 4)
    {
      mySerial.write(0x04);
      mySerial.write(0xEF);
    }
  }
  else if(output == 4)
  {
    mySerial.write(0x04);
    if(input == 1)
    {
      mySerial.write(0x01);
      mySerial.write(0xBE);
    }
    else if(input == 2)
    {
      mySerial.write(0x02);
      mySerial.write(0x5C);
    }
    else if(input == 3)
    {
      mySerial.write(0x03);
      mySerial.write(0x02);
    }
    else if(input == 4)
    {
      mySerial.write(0x04);
      mySerial.write(0x81);
    }
  }    
}



